@@ -18,8 +18,6 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
         if self.data:
             self.impuestos_trasladados = self.data.get('impuestos_trasladados', [])
             self.impuestos_retenidos = self.data.get('impuestos_retenidos', [])
-            self.impuestos_trasladados_p = self.data.get('impuestos_trasladados_p', [])
-            self.impuestos_retenidos_p = self.data.get('impuestos_retenidos_p', [])
 
     def generate_xml(self):
         self.reset_dom()
@@ -156,7 +154,7 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
         try:
             emisor = self.dom.createElement('cfdi:Emisor')
             emisor.setAttribute('Rfc', self.data.get('emisor_rfc'))
-            emisor.setAttribute('Nombre', replace(self.data.get('emisor_nombre', '')).strip())
+            emisor.setAttribute('Nombre', self.data.get('emisor_nombre', '').strip())
             emisor.setAttribute('RegimenFiscal', self.data.get('regimen_fiscal'))
             return emisor
         except Exception as e:
@@ -166,7 +164,7 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
         try:
             receptor = self.dom.createElement('cfdi:Receptor')
             receptor.setAttribute('Rfc', self.data.get('receptor_rfc'))
-            receptor.setAttribute('Nombre',replace(self.data.get('receptor_nombre', '')).strip())
+            receptor.setAttribute('Nombre',self.data.get('receptor_nombre', '').strip())
             receptor.setAttribute('DomicilioFiscalReceptor', self.data.get('domicilio_fiscal_receptor', ''))
             if self.data.get('receptor_residencia_fiscal') != 'MEX':
                 receptor.setAttribute('ResidenciaFiscal', self.data.get('receptor_residencia_fiscal'))
@@ -279,51 +277,47 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
 
     def get_complemento_pagos(self):
         try:
+            complemento_data = self.data.get('complemento_pago')
             pagos = self.dom.createElement('pago20:Pagos')
-            pagos.setAttribute('Version','2.0')
+            pagos.setAttribute('Version', complemento_data.get('version'))
 
-            totales = self.dom.createElement('pago20:Totales')
-            totales.setAttribute('MontoTotalPagos', str(self.data.get('monto')))
-
-            for imp_ret in self.data.get('impuestos_trasladados_p'):
-                if imp_ret.get('impuesto') == '002' and imp_ret.get('tipo_factor') == 'Exento':
-                    totales.setAttribute('TotalTrasladosBaseIVAExento', str(imp_ret.get('base')))
+            totales_pago = self.get_totales_pago(complemento_data)
 
             pago = self.dom.createElement('pago20:Pago')
-            pago.setAttribute('FechaPago', self.data.get('fecha_pago'))
+            pago.setAttribute('FechaPago', complemento_data.get('fecha_pago'))
 
-            if self.data.get('forma_pago') == '99':
+            if complemento_data.get('forma_pago') == '99':
                 raise Exception('La forma de pago no puede ser "Por Definir".')
 
-            pago.setAttribute('FormaDePagoP', self.data.get('forma_pago'))
-            pago.setAttribute('MonedaP', self.data.get('moneda_pago'))
-            pago.setAttribute('TipoCambioP', self.data.get('tipo_cambio'))
-            pago.setAttribute('Monto', str(self.data.get('monto')))
+            pago.setAttribute('FormaDePagoP', complemento_data.get('forma_pago'))
+            pago.setAttribute('MonedaP', complemento_data.get('moneda'))
+            pago.setAttribute('TipoCambioP', complemento_data.get('tipo_cambio'))
+            pago.setAttribute('Monto', str(complemento_data.get('monto')))
 
-            if self.data.get('numero_operacion') != '':
-                pago.setAttribute('NumOperacion', self.data.get('numero_operacion'))
+            if complemento_data.get('numero_operacion'):
+                pago.setAttribute('NumOperacion', complemento_data.get('numero_operacion'))
 
-            if self.data.get('bancarizado'):
-                if self.data.get('patron_cuenta_ordenante') and self.data.get('patron_cuenta_ordenante') != '':
-                    if self.data.get('banco_ordenante'):
-                        pago.setAttribute('RfcEmisorCtaOrd', self.data.get('emisor_cuenta_ordenante_rfc'))
-                        if self.data.get('emisor_cuenta_ordenante_rfc') == 'XEXX010101000':
-                            pago.setAttribute('NomBancoOrdExt', self.data.get('emisor_cuenta_ordenante_nombre'))
-                    pago.setAttribute('CtaOrdenante', self.data.get('emisor_cuenta_ordenante'))
-                if self.data.get('forma_pago') != '06':
-                    if self.data.get('emisor_cuenta_beneficiaria_rfc'):
-                        pago.setAttribute('RfcEmisorCtaBen', self.data.get('emisor_cuenta_beneficiaria_rfc'))
-                    pago.setAttribute('CtaBeneficiario', self.data.get('emisor_cuenta_beneficiaria'))
+            if complemento_data.get('bancarizado'):
+                if complemento_data.get('patron_cuenta_ordenante') and complemento_data.get('patron_cuenta_ordenante') != '':
+                    if complemento_data.get('banco_ordenante'):
+                        pago.setAttribute('RfcEmisorCtaOrd', complemento_data.get('emisor_cuenta_ordenante_rfc'))
+                        if complemento_data.get('emisor_cuenta_ordenante_rfc') == 'XEXX010101000':
+                            pago.setAttribute('NomBancoOrdExt', complemento_data.get('emisor_cuenta_ordenante_nombre'))
+                    pago.setAttribute('CtaOrdenante', complemento_data.get('emisor_cuenta_ordenante'))
+                if complemento_data.get('forma_pago') != '06':
+                    if complemento_data.get('emisor_cuenta_beneficiaria_rfc'):
+                        pago.setAttribute('RfcEmisorCtaBen', complemento_data.get('emisor_cuenta_beneficiaria_rfc'))
+                    pago.setAttribute('CtaBeneficiario', complemento_data.get('emisor_cuenta_beneficiaria'))
 
-            doctos = self.data.get('doctos_relacionados')
+            doctos = complemento_data.get('doctos_relacionados', [])
             for docto_relacionado in doctos:
                 docto = self.dom.createElement('pago20:DoctoRelacionado')
                 docto.setAttribute('IdDocumento', docto_relacionado.get('uuid'))
                 docto.setAttribute('Serie', docto_relacionado.get('serie'))
                 docto.setAttribute('Folio', docto_relacionado.get('folio'))
                 docto.setAttribute('MonedaDR', docto_relacionado.get('moneda'))
-                docto.setAttribute('EquivalenciaDR', docto_relacionado.get('tipo_cambio'))
-                docto.setAttribute('NumParcialidad', str(docto_relacionado.get('num_parcialidad')))
+                docto.setAttribute('EquivalenciaDR', docto_relacionado.get('equivalencia'))
+                docto.setAttribute('NumParcialidad', str(docto_relacionado.get('parcialidad')))
                 docto.setAttribute('ImpSaldoAnt', str(docto_relacionado.get('imp_saldo_ant')))
                 docto.setAttribute('ImpPagado', str(docto_relacionado.get('imp_pagado')))
                 docto.setAttribute('ImpSaldoInsoluto', str(docto_relacionado.get('imp_saldo_insoluto')))
@@ -332,26 +326,55 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
                 if docto_relacionado.get('objeto_impuesto', '01') != '01':
                     docto.appendChild(self.get_impuestos_documentos_relacionados_pago(docto_relacionado))
                 pago.appendChild(docto)
-            pagos.appendChild(totales)
+            pagos.appendChild(totales_pago)
             pagos.appendChild(pago)
-            if len(self.data.get('impuestos_retenidos_p')) > 0 or len(self.data.get('impuestos_trasladados_p')):
-                pago.appendChild(self.get_impuestos_pago())
+            if len(complemento_data.get('impuestos_retenidos', [])) > 0 or len(complemento_data.get('impuestos_trasladados', [])):
+                pago.appendChild(self.get_impuestos_pago(complemento_data))
             pagos.appendChild(pago)
             return pagos
         except Exception as e:
             raise Exception("Ocurrio un error al agregar el complemento de pago. Error::: " + str(e))
 
+    def get_totales_pago(self, complemento_data):
+        totales = self.dom.createElement('pago20:Totales')
+
+        if complemento_data.get('total_retenciones_iva'):
+            totales.setAttribute('TotalRetencionesIVA', str(complemento_data.get('total_retenciones_iva')))
+        if complemento_data.get('total_retenciones_isr'):
+            totales.setAttribute('TotalRetencionesISR', str(complemento_data.get('total_retenciones_isr')))
+        if complemento_data.get('total_retenciones_ieps'):
+            totales.setAttribute('TotalRetencionesIEPS', str(complemento_data.get('total_retenciones_ieps')))
+        if complemento_data.get('total_traslados_base_iva16'):
+            totales.setAttribute('TotalTrasladosBaseIVA16', str(complemento_data.get('total_traslados_base_iva16')))
+        if complemento_data.get('total_traslados_impuesto_iva16'):
+            totales.setAttribute('TotalTrasladosImpuestoIVA16', str(complemento_data.get('total_traslados_impuesto_iva16')))
+        if complemento_data.get('total_traslados_base_iva8'):
+            totales.setAttribute('TotalTrasladosBaseIVA8', str(complemento_data.get('total_traslados_base_iva8')))
+        if complemento_data.get('total_traslados_impuesto_iva8'):
+            totales.setAttribute('TotalTrasladosImpuestoIVA8', str(complemento_data.get('total_traslados_impuesto_iva8')))
+        if complemento_data.get('total_traslados_base_iva0'):
+            totales.setAttribute('TotalTrasladosBaseIVA0', str(complemento_data.get('total_traslados_base_iva0')))
+        if complemento_data.get('total_traslados_impuesto_iva0'):
+            totales.setAttribute('TotalTrasladosImpuestoIVA0', str(complemento_data.get('total_traslados_impuesto_iva0')))
+        if complemento_data.get('total_traslados_base_iva_exento'):
+            totales.setAttribute('TotalTrasladosBaseIVAExento', str(complemento_data.get('total_traslados_base_iva_exento')))
+
+        totales.setAttribute('MontoTotalPagos', str(complemento_data.get('monto')))
+
+        return totales
+
+    # ***** Impuestos Pago *****
     def get_impuestos_documentos_relacionados_pago(self, docto_relacionado):
         impuestos = self.dom.createElement('pago20:ImpuestosDR')
-        if len(docto_relacionado.get('retenciones', [])) > 0:
+        if len(docto_relacionado.get('impuestos_retenidos', [])) > 0:
             retenciones = self.dom.createElement('pago20:RetencionesDR')
-            for retencion in docto_relacionado.get('retenciones', []):
+            for retencion in docto_relacionado.get('impuestos_retenidos', []):
                 retenciones.appendChild(self.get_retenciones_documentos_relacionados(retencion))
             impuestos.appendChild(retenciones)
 
-        if len(docto_relacionado.get('traslados', [])) > 0:
+        if len(docto_relacionado.get('impuestos_trasladados', [])) > 0:
             traslados = self.dom.createElement('pago20:TrasladosDR')
-            for traslado in docto_relacionado.get('traslados', []):
+            for traslado in docto_relacionado.get('impuestos_trasladados', []):
                 traslados.appendChild(self.get_traslados_documentos_relacionados(traslado))
             impuestos.appendChild(traslados)
         return impuestos
@@ -370,26 +393,23 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
         traslado.setAttribute('BaseDR', str(traslado_data.get('base')))  # Importe del documentos_relacionados
         traslado.setAttribute('ImpuestoDR', traslado_data.get('impuesto', ''))
         traslado.setAttribute('TipoFactorDR', traslado_data.get('tipo_factor', 'Tasa'))  # Tipo factor Tasa ó Cuota
-        if traslado_data.get('tasa_cuota'):
+        if traslado_data.get('tipo_factor') in ['Tasa', 'Cuota']:
             traslado.setAttribute('TasaOCuotaDR', str(traslado_data.get('tasa_cuota')))  # valor del impuesto que se envia
-        if traslado_data.get('importe'):
             traslado.setAttribute('ImporteDR', str(traslado_data.get('importe')))
         return traslado
 
-    def get_impuestos_pago(self):
+    def get_impuestos_pago(self, complemento_data):
         try:
             impuestos_p = self.dom.createElement('pago20:ImpuestosP')
-            if len(self.impuestos_retenidos_p) > 0:
+            if len(complemento_data.get('impuestos_retenidos', [])) > 0:
                 retenciones = self.dom.createElement('pago20:RetencionesP')
-                total_retenciones = 0
-                for retencion in self.impuestos_retenidos_p:
-                    total_retenciones += float(retencion.get('importe', 0))
-                    retenciones.appendChild(self.get_retenciones(retencion))
+                for retencion_data in complemento_data.get('impuestos_retenidos'):
+                    retenciones.appendChild(self.get_retenciones_pago(retencion_data))
                 impuestos_p.appendChild(retenciones)
-            if len(self.impuestos_trasladados_p) > 0:
+            if len(complemento_data.get('impuestos_trasladados', [])) > 0:
                 traslados = self.dom.createElement('pago20:TrasladosP')
-                for traslado in self.impuestos_trasladados_p:
-                    traslados.appendChild(self.get_traslados_pago(traslado))
+                for traslado_data in complemento_data.get('impuestos_trasladados'):
+                    traslados.appendChild(self.get_traslados_pago(traslado_data))
                 impuestos_p.appendChild(traslados)
             return impuestos_p
         except Exception as e:
@@ -411,6 +431,7 @@ class GeneratorXmlFourDotZero(AbstractGeneratorXml):
         retencion.setAttribute('ImpuestoP', retencion_data.get('impuesto'))
         retencion.setAttribute('ImporteP', str(retencion_data.get('importe')))
         return retencion
+    # ***** Impuestos Pago *****
 
     def get_addenda(self):
         try:
